@@ -1,6 +1,6 @@
 /* arrayfunc.h -- declarations for miscellaneous array functions in arrayfunc.c */
 
-/* Copyright (C) 2001-2009 Free Software Foundation, Inc.
+/* Copyright (C) 2001-2010 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -25,9 +25,18 @@
 
 #if defined (ARRAY_VARS)
 
+/* Flags for array_value_internal and callers array_value/get_array_value */
+#define AV_ALLOWALL	0x001
+#define AV_QUOTED	0x002
+#define AV_USEIND	0x004
+#define AV_USEVAL	0x008	/* XXX - should move this */
+#define AV_ASSIGNRHS	0x010	/* no splitting, special case ${a[@]} */
+
 extern SHELL_VAR *convert_var_to_array __P((SHELL_VAR *));
 extern SHELL_VAR *convert_var_to_assoc __P((SHELL_VAR *));
 
+extern char *make_array_variable_value __P((SHELL_VAR *, arrayind_t, char *, char *, int));
+
 extern SHELL_VAR *bind_array_variable __P((char *, arrayind_t, char *, int));
 extern SHELL_VAR *bind_array_element __P((SHELL_VAR *, arrayind_t, char *, int));
 extern SHELL_VAR *assign_array_element __P((char *, char *, int));
@@ -49,15 +58,22 @@ extern int skipsubscript __P((const char *, int, int));
 extern void print_array_assignment __P((SHELL_VAR *, int));
 extern void print_assoc_assignment __P((SHELL_VAR *, int));
 
-extern arrayind_t array_expand_index __P((char *, int));
-extern int valid_array_reference __P((char *));
-extern char *array_value __P((char *, int, int *));
-extern char *get_array_value __P((char *, int, int *));
+extern arrayind_t array_expand_index __P((SHELL_VAR *, char *, int));
+extern int valid_array_reference __P((const char *, int));
+extern char *array_value __P((const char *, int, int, int *, arrayind_t *));
+extern char *get_array_value __P((const char *, int, int *, arrayind_t *));
 
 extern char *array_keys __P((char *, int));
 
-extern char *array_variable_name __P((char *, char **, int *));
-extern SHELL_VAR *array_variable_part __P((char *, char **, int *));
+extern char *array_variable_name __P((const char *, char **, int *));
+extern SHELL_VAR *array_variable_part __P((const char *, char **, int *));
+
+#else
+
+#define AV_ALLOWALL	0
+#define AV_QUOTED	0
+#define AV_USEIND	0
+#define AV_ASSIGNRHS	0
 
 #endif
 
