@@ -1,29 +1,30 @@
+/* isearch.c - incremental searching */
+
 /* **************************************************************** */
 /*								    */
 /*			I-Search and Searching			    */
 /*								    */
 /* **************************************************************** */
 
-/* Copyright (C) 1987-2005 Free Software Foundation, Inc.
+/* Copyright (C) 1987-2015 Free Software Foundation, Inc.
 
-   This file contains the Readline Library (the Library), a set of
-   routines for providing Emacs style line input to programs that ask
-   for it.
+   This file is part of the GNU Readline Library (Readline), a library
+   for reading lines of text with interactive input and history editing.      
 
-   The Library is free software; you can redistribute it and/or modify
+   Readline is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
-   The Library is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   Readline is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with Readline.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
-   The GNU General Public License is often shipped with GNU software, and
-   is generally kept in a file called COPYING or LICENSE.  If you do not
-   have a copy of the license, write to the Free Software Foundation,
-   59 Temple Place, Suite 330, Boston, MA 02111 USA. */
 #define READLINE_LIBRARY
 
 #if defined (HAVE_CONFIG_H)
@@ -65,7 +66,6 @@ static int rl_search_history PARAMS((int, int));
 
 static _rl_search_cxt *_rl_isearch_init PARAMS((int));
 static void _rl_isearch_fini PARAMS((_rl_search_cxt *));
-static int _rl_isearch_cleanup PARAMS((_rl_search_cxt *, int));
 
 /* Last line found by the current incremental search, so we don't `find'
    identical lines many times in a row.  Now part of isearch context. */
@@ -75,7 +75,7 @@ static int _rl_isearch_cleanup PARAMS((_rl_search_cxt *, int));
 static char *last_isearch_string;
 static int last_isearch_string_len;
 
-static char *default_isearch_terminators = "\033\012";
+static char * const default_isearch_terminators = "\033\012";
 
 _rl_search_cxt *
 _rl_scxt_alloc (type, flags)
@@ -103,10 +103,13 @@ _rl_scxt_alloc (type, flags)
 
   cxt->save_undo_list = 0;
 
+  cxt->keymap = _rl_keymap;
+  cxt->okeymap = _rl_keymap;
+
   cxt->history_pos = 0;
   cxt->direction = 0;
 
-  cxt->lastc = 0;
+  cxt->prevc = cxt->lastc = 0;
 
   cxt->sline = 0;
   cxt->sline_len = cxt->sline_index = 0;
@@ -125,7 +128,7 @@ _rl_scxt_dispose (cxt, flags)
   FREE (cxt->allocated_line);
   FREE (cxt->lines);
 
-  free (cxt);
+  xfree (cxt);
 }
 
 /* Search backwards through the history looking for a string which is typed
@@ -152,16 +155,16 @@ rl_forward_search_history (sign, key)
    WHERE is the history list number of the current line.  If it is
    -1, then this line is the starting one. */
 static void
-rl_display_search (search_string, reverse_p, where)
+rl_display_search (search_string, flags, where)
      char *search_string;
-     int reverse_p, where;
+     int flags, where;
 {
   char *message;
   int msglen, searchlen;
 
   searchlen = (search_string && *search_string) ? strlen (search_string) : 0;
 
-  message = (char *)xmalloc (searchlen + 33);
+  message = (char *)xmalloc (searchlen + 64);
   msglen = 0;
 
 #if defined (NOTDEF)
@@ -174,7 +177,13 @@ rl_display_search (search_string, reverse_p, where)
 
   message[msglen++] = '(';
 
-  if (reverse_p)
+  if (flags & SF_FAILED)
+    {
+      strcpy (message + msglen, "failed ");
+      msglen += 7;
+    }
+
+  if (flags & SF_REVERSE)
     {
       strcpy (message + msglen, "reverse-");
       msglen += 8;
@@ -192,7 +201,7 @@ rl_display_search (search_string, reverse_p, where)
   strcpy (message + msglen, "': ");
 
   rl_message ("%s", message);
-  free (message);
+  xfree (message);
   (*rl_redisplay_function) ();
 }
 
@@ -211,7 +220,7 @@ _rl_isearch_init (direction)
   cxt->search_terminators = _rl_isearch_terminators ? _rl_isearch_terminators
 						: default_isearch_terminators;
 
-  /* Create an arrary of pointers to the lines that we want to search. */
+  /* Create an array of pointers to the lines that we want to search. */
   hlist = history_list ();
   rl_maybe_replace_line ();
   i = 0;
@@ -308,13 +317,19 @@ _rl_search_getchar (cxt)
   RL_UNSETSTATE(RL_STATE_MOREINPUT);
 
 #if defined (HANDLE_MULTIBYTE)
+  /* This ends up with C (and LASTC) being set to the last byte of the
+     multibyte character.  In most cases c == lastc == mb[0] */
   if (MB_CUR_MAX > 1 && rl_byte_oriented == 0)
     c = cxt->lastc = _rl_read_mbstring (cxt->lastc, cxt->mb, MB_LEN_MAX);
 #endif
 
+  RL_CHECK_SIGNALS ();
   return c;
 }
 
+#define ENDSRCH_CHAR(c) \
+  ((CTRL_CHAR (c) || META_CHAR (c) || (c) == RUBOUT) && ((c) != CTRL ('G')))
+
 /* Process just-read character C according to isearch context CXT.  Return
    -1 if the caller should just free the context and return, 0 if we should
    break out of the loop, and 1 if we should continue to read characters. */
@@ -327,11 +342,60 @@ _rl_isearch_dispatch (cxt, c)
   rl_command_func_t *f;
 
   f = (rl_command_func_t *)NULL;
- 
- /* Translate the keys we do something with to opcodes. */
-  if (c >= 0 && _rl_keymap[c].type == ISFUNC)
+
+  if (c < 0)
     {
-      f = _rl_keymap[c].function;
+      cxt->sflags |= SF_FAILED;
+      cxt->history_pos = cxt->last_found_line;
+      return -1;
+    }
+
+  /* If we are moving into a new keymap, modify cxt->keymap and go on.
+     This can be a problem if c == ESC and we want to terminate the
+     incremental search, so we check */
+  if (c >= 0 && cxt->keymap[c].type == ISKMAP && strchr (cxt->search_terminators, cxt->lastc) == 0)
+    {
+      /* _rl_keyseq_timeout specified in milliseconds; _rl_input_queued
+	 takes microseconds, so multiply by 1000.  If we don't get any
+	 additional input and this keymap shadows another function, process
+	 that key as if it was all we read. */
+      if (_rl_keyseq_timeout > 0 &&
+	    RL_ISSTATE (RL_STATE_CALLBACK) == 0 &&
+	    RL_ISSTATE (RL_STATE_INPUTPENDING) == 0 &&
+	    _rl_pushed_input_available () == 0 &&
+	    ((Keymap)(cxt->keymap[c].function))[ANYOTHERKEY].function &&
+	    _rl_input_queued (_rl_keyseq_timeout*1000) == 0)
+	goto add_character;
+
+      cxt->okeymap = cxt->keymap;
+      cxt->keymap = FUNCTION_TO_KEYMAP (cxt->keymap, c);
+      cxt->sflags |= SF_CHGKMAP;
+      /* XXX - we should probably save this sequence, so we can do
+	 something useful if this doesn't end up mapping to a command we
+	 interpret here.  Right now we just save the most recent character
+	 that caused the index into a new keymap. */
+      cxt->prevc = c;
+#if defined (HANDLE_MULTIBYTE)
+      if (MB_CUR_MAX > 1 && rl_byte_oriented == 0)
+	{
+	  if (cxt->mb[1] == 0)
+	    {
+	      cxt->pmb[0] = c;		/* XXX should be == cxt->mb[0] */
+	      cxt->pmb[1] = '\0';
+	    }
+	  else
+	    memcpy (cxt->pmb, cxt->mb, sizeof (cxt->pmb));
+	}
+#endif
+      return 1;
+    }
+
+add_character:
+
+  /* Translate the keys we do something with to opcodes. */
+  if (c >= 0 && cxt->keymap[c].type == ISFUNC)
+    {
+      f = cxt->keymap[c].function;
 
       if (f == rl_reverse_search_history)
 	cxt->lastc = (cxt->sflags & SF_REVERSE) ? -1 : -2;
@@ -339,19 +403,75 @@ _rl_isearch_dispatch (cxt, c)
 	cxt->lastc = (cxt->sflags & SF_REVERSE) ? -2 : -1;
       else if (f == rl_rubout)
 	cxt->lastc = -3;
-      else if (c == CTRL ('G'))
+      else if (c == CTRL ('G') || f == rl_abort)
 	cxt->lastc = -4;
-      else if (c == CTRL ('W'))	/* XXX */
+      else if (c == CTRL ('W') || f == rl_unix_word_rubout)	/* XXX */
 	cxt->lastc = -5;
-      else if (c == CTRL ('Y'))	/* XXX */
+      else if (c == CTRL ('Y') || f == rl_yank)	/* XXX */
 	cxt->lastc = -6;
     }
 
+  /* If we changed the keymap earlier while translating a key sequence into
+     a command, restore it now that we've succeeded. */
+  if (cxt->sflags & SF_CHGKMAP)
+    {
+      cxt->keymap = cxt->okeymap;
+      cxt->sflags &= ~SF_CHGKMAP;
+      /* If we indexed into a new keymap, but didn't map to a command that
+	 affects the search (lastc > 0), and the character that mapped to a
+	 new keymap would have ended the search (ENDSRCH_CHAR(cxt->prevc)),
+	 handle that now as if the previous char would have ended the search
+	 and we would have read the current character. */
+      /* XXX - should we check cxt->mb? */
+      if (cxt->lastc > 0 && ENDSRCH_CHAR (cxt->prevc))
+	{
+	  rl_stuff_char (cxt->lastc);
+	  rl_execute_next (cxt->prevc);
+	  /* XXX - do we insert everything in cxt->pmb? */
+	  return (0);
+	}
+      /* Otherwise, if the current character is mapped to self-insert or
+	 nothing (i.e., not an editing command), and the previous character
+	 was a keymap index, then we need to insert both the previous
+	 character and the current character into the search string. */
+      else if (cxt->lastc > 0 && cxt->prevc > 0 &&
+	       cxt->keymap[cxt->prevc].type == ISKMAP &&
+	       (f == 0 || f == rl_insert))
+	{
+	  /* Make lastc be the next character read */
+	  /* XXX - do we insert everything in cxt->mb? */
+	  rl_execute_next (cxt->lastc);
+	  /* Dispatch on the previous character (insert into search string) */
+	  cxt->lastc = cxt->prevc;
+#if defined (HANDLE_MULTIBYTE)
+	  /* Have to overwrite cxt->mb here because dispatch uses it below */
+	  if (MB_CUR_MAX > 1 && rl_byte_oriented == 0)
+	    {  
+	      if (cxt->pmb[1] == 0)	  
+		{
+		  cxt->mb[0] = cxt->lastc;	/* == cxt->prevc */
+		  cxt->mb[1] = '\0';
+		}
+	      else
+		memcpy (cxt->mb, cxt->pmb, sizeof (cxt->mb));
+	    }
+#endif
+	  cxt->prevc = 0;	  
+	}
+      else if (cxt->lastc > 0 && cxt->prevc > 0 && f && f != rl_insert)
+	{
+	  rl_stuff_char (cxt->lastc);
+	  rl_execute_next (cxt->prevc);
+	  /* XXX - do we insert everything in cxt->pmb? */
+	  return (0);
+	}
+    }
+
   /* The characters in isearch_terminators (set from the user-settable
      variable isearch-terminators) are used to terminate the search but
      not subsequently execute the character as a command.  The default
      value is "\033\012" (ESC and C-J). */
-  if (strchr (cxt->search_terminators, cxt->lastc))
+  if (cxt->lastc > 0 && strchr (cxt->search_terminators, cxt->lastc))
     {
       /* ESC still terminates the search, but if there is pending
 	 input or if input arrives within 0.1 seconds (on systems
@@ -362,20 +482,17 @@ _rl_isearch_dispatch (cxt, c)
 	 XXX - since _rl_input_available depends on the application-
 	 settable keyboard timeout value, this could alternatively
 	 use _rl_input_queued(100000) */
-      if (cxt->lastc == ESC && _rl_input_available ())
+      if (cxt->lastc == ESC && (_rl_pushed_input_available () || _rl_input_available ()))
 	rl_execute_next (ESC);
       return (0);
     }
 
-#define ENDSRCH_CHAR(c) \
-  ((CTRL_CHAR (c) || META_CHAR (c) || (c) == RUBOUT) && ((c) != CTRL ('G')))
-
 #if defined (HANDLE_MULTIBYTE)
   if (MB_CUR_MAX > 1 && rl_byte_oriented == 0)
     {
       if (cxt->lastc >= 0 && (cxt->mb[0] && cxt->mb[1] == '\0') && ENDSRCH_CHAR (cxt->lastc))
 	{
-	  /* This sets rl_pending_input to c; it will be picked up the next
+	  /* This sets rl_pending_input to LASTC; it will be picked up the next
 	     time rl_read_key is called. */
 	  rl_execute_next (cxt->lastc);
 	  return (0);
@@ -405,7 +522,7 @@ _rl_isearch_dispatch (cxt, c)
 	      cxt->search_string = (char *)xrealloc (cxt->search_string, cxt->search_string_size);
 	      strcpy (cxt->search_string, last_isearch_string);
 	      cxt->search_string_index = last_isearch_string_len;
-	      rl_display_search (cxt->search_string, (cxt->sflags & SF_REVERSE), -1);
+	      rl_display_search (cxt->search_string, cxt->sflags, -1);
 	      break;
 	    }
 	  return (1);
@@ -435,8 +552,20 @@ _rl_isearch_dispatch (cxt, c)
 	 do until we have a real isearch-undo. */
       if (cxt->search_string_index == 0)
 	rl_ding ();
-      else
+      else if (MB_CUR_MAX == 1 || rl_byte_oriented)
 	cxt->search_string[--cxt->search_string_index] = '\0';
+      else
+	{
+	  wstart = _rl_find_prev_mbchar (cxt->search_string, cxt->search_string_index, MB_FIND_NONZERO);
+	  if (wstart >= 0)
+	    cxt->search_string[cxt->search_string_index = wstart] = '\0';
+	  else
+	    cxt->search_string[cxt->search_string_index = 0] = '\0';
+	}
+
+      if (cxt->search_string_index == 0)
+	rl_ding ();
+
       break;
 
     case -4:	/* C-G, abort */
@@ -513,18 +642,28 @@ _rl_isearch_dispatch (cxt, c)
       if (MB_CUR_MAX > 1 && rl_byte_oriented == 0)
 	{
 	  int j, l;
-	  for (j = 0, l = strlen (cxt->mb); j < l; )
-	    cxt->search_string[cxt->search_string_index++] = cxt->mb[j++];
+
+	  if (cxt->mb[0] == 0 || cxt->mb[1] == 0)
+	    cxt->search_string[cxt->search_string_index++] = cxt->mb[0];
+	  else
+	    for (j = 0, l = RL_STRLEN (cxt->mb); j < l; )
+	      cxt->search_string[cxt->search_string_index++] = cxt->mb[j++];
 	}
       else
 #endif
-	cxt->search_string[cxt->search_string_index++] = c;
+	cxt->search_string[cxt->search_string_index++] = cxt->lastc;	/* XXX - was c instead of lastc */
       cxt->search_string[cxt->search_string_index] = '\0';
       break;
     }
 
   for (cxt->sflags &= ~(SF_FOUND|SF_FAILED);; )
     {
+      if (cxt->search_string_index == 0)
+	{
+	  cxt->sflags |= SF_FAILED;
+	  break;
+	}
+
       limit = cxt->sline_len - cxt->search_string_index + 1;
 
       /* Search the current line. */
@@ -575,6 +714,7 @@ _rl_isearch_dispatch (cxt, c)
       /* We cannot find the search string.  Ding the bell. */
       rl_ding ();
       cxt->history_pos = cxt->last_found_line;
+      rl_display_search (cxt->search_string, cxt->sflags, (cxt->history_pos == cxt->save_line) ? -1 : cxt->history_pos);
       return 1;
     }
 
@@ -587,13 +727,13 @@ _rl_isearch_dispatch (cxt, c)
       rl_replace_line (cxt->lines[cxt->history_pos], 0);
       rl_point = cxt->sline_index;
       cxt->last_found_line = cxt->history_pos;
-      rl_display_search (cxt->search_string, (cxt->sflags & SF_REVERSE), (cxt->history_pos == cxt->save_line) ? -1 : cxt->history_pos);
+      rl_display_search (cxt->search_string, cxt->sflags, (cxt->history_pos == cxt->save_line) ? -1 : cxt->history_pos);
     }
 
   return 1;
 }
 
-static int
+int
 _rl_isearch_cleanup (cxt, r)
      _rl_search_cxt *cxt;
      int r;
@@ -622,7 +762,7 @@ rl_search_history (direction, invoking_key)
   RL_SETSTATE(RL_STATE_ISEARCH);
   cxt = _rl_isearch_init (direction);
 
-  rl_display_search (cxt->search_string, (cxt->sflags & SF_REVERSE), -1);
+  rl_display_search (cxt->search_string, cxt->sflags, -1);
 
   /* If we are using the callback interface, all we do is set up here and
       return.  The key is that we leave RL_STATE_ISEARCH set. */
